////////////////////////////////////////////////////////////////////////////
// CTextEdit 饹Υץơ
//
////////////////////////////////////////////////////////////////////////////

#include "TextEdit.h"
#include "TaEditDoc.h"
#include "TaEditDraw.h"
#include "TaEditShell.h"
#include "TaEditJPEXT.h"
#include "TaEditMovetoDlg.h"
#include "TaEditProcessingDlg.h"
#include "FindText.h"
#include "ReplaceAll.h"

////////////////////////////////////////////////////////////////////////////
// /˴
////////////////////////////////////////////////////////////////////////////

CTextEdit::CTextEdit( CTaEditDraw *argpDraw, VClsPtr< CCursor > argpCursorInfo, CTaEditDoc *argpDoc ) :
	pDraw( argpDraw ),
	pCursorInfo( argpCursorInfo ),
	pDoc( argpDoc )
{

}

CTextEdit::~CTextEdit()
{

}

////////////////////////////////////////////////////////////////////////////
// ᥽å
////////////////////////////////////////////////////////////////////////////

// 򤵤ƤϰϤ˥ƥȤĥդ
void CTextEdit::PasteText( const wstring& rStr, bool DoRedraw )
{
	T_CurPos NextPos;
	ScrollInfoPacket ScrPacket;
	T_SelRange SR = pCursorInfo->GetSelRange();
	T_CurPos CP = pCursorInfo->GetCurPos();

	// ʸ
	m_TaEditShell->ProcErrorMsg( pDoc->Insert( rStr, SR.GetSPos(), SR, &NextPos ) );

	// 
	pCursorInfo->SetCurPos( NextPos, &ScrPacket, T_SelRange( NextPos, NextPos ) );

	// 
	if ( DoRedraw ) pDraw->AfterEditRedraw( ScrPacket );
}

// ɥ
void CTextEdit::Undo()
{
	ScrollInfoPacket scrPacket;
	T_SelRange wSR;	// ɥϰ
	T_CurPos wCP;	// ɥΥ

	ClearSelect();	// öϰ롣

	pDoc->Undo( &wSR, &wCP );
	pCursorInfo->SetCurPos( wCP, &scrPacket, wSR );	// ֤
	pDraw->AfterEditRedraw( scrPacket );	// 
}

// ɥ
void CTextEdit::Redo()
{
	ScrollInfoPacket scrPacket;
	T_SelRange wSR;	// ɥϰ
	T_CurPos wCP;	// ɥΥ

	ClearSelect();	// öϰ롣

	pDoc->Redo( &wSR, &wCP );
	pCursorInfo->SetCurPos( wCP, &scrPacket, wSR );	// ֤
	pDraw->AfterEditRedraw( scrPacket );	// 
}

// ϰϤκ
void CTextEdit::DelSelRange()
{
	ScrollInfoPacket scrPacket;
	T_CurPos wCP;

	// 򤵤ƤϰϤ
	CErrorBool r = pDoc->InsertSpecialCode( XK_Delete, pCursorInfo->GetCurPos(), pCursorInfo->GetSelRange(), &wCP );
	m_TaEditShell->ProcErrorMsg( r );

	// ֤
	pCursorInfo->SetCurPos( wCP, &scrPacket, false );

	// 
	pDraw->AfterEditRedraw( scrPacket );
}

// 
void CTextEdit::FullSelect()
{
	ScrollInfoPacket scrPacket;

	// ߤΥ֤
	T_CurPos wCP = pCursorInfo->GetCurPos();

	// Կ
	const int LineCnt = pDoc->RefData().size();

	// ǸιԤ˴ޤޤʸ
	T_LineDataItr_c LastLineItr = pDoc->RefData().end();
	LastLineItr--;
	const int LastLineCharCnt = LastLineItr->length();

	// ꤹ٤ϰϤ
	T_SelRange wSR( T_CurPos( 0, 0 ), T_CurPos( LineCnt - 1, LastLineCharCnt ) );

	// ֤
	pCursorInfo->SetCurPos( wCP, &scrPacket, wSR );

	// 
	pDraw->AfterSelRangeRedraw( scrPacket.ChangeSLine, scrPacket.ChangeELine );
}

// å
void CTextEdit::Cut()
{
	if ( !IsEnableCopy() ) return ;

	Copy();	// ߤϰϤ򥳥ԡ
	DelSelRange();	// ϰϤ
}

// ԡ
void CTextEdit::Copy()
{
	T_SelRange SR = pCursorInfo->GetSelRange();
	wstring wStr;

	if ( !IsEnableCopy() ) return ;

	// ԡоݤʸ
	pDoc->GetRangeStr( SR, &wStr );

	// åץܡɤ
	JPEXT_SetClipboardData( m_TaEditShell->m_TaEditShell, wStr );
}

// ڡ
void CTextEdit::Paste()
{
	wstring wStr;

	// åץܡɤǡ
	JPEXT_GetClipboardData( m_TaEditShell->m_TaEditShell, &wStr );

	// ƥȤ
	PasteText( wStr, true );
}

// åץܡɤؤΥԡǽݤ
bool CTextEdit::IsEnableCopy() const
{
	if ( pCursorInfo->GetSelRange().GetLength() <= T_CurPos( 0, 0 ) )
		return false;
	else
		return true;
}

// ߤΥ֤ѥΥƥȤȰפʬ򸡺롣
void CTextEdit::FindText( const wstring &rPattern )
{
	ScrollInfoPacket scrPacket;
	T_SelRange nSelRange;
	T_CurPos nCurPos;
	VClsPtr< CConfigInfo > pConf = m_TaEditShell->GetConfigInfo();
	int ShowDlgDelay = pConf->GetShowPrecessingDlgDelay();
	CFindText *pFindTextObj = new CFindText();
	if ( NULL == pFindTextObj ) {
		m_TaEditShell->ShowErrorMsgBox( pConf->RefOutOfMemoryErrorMsg() );
		return ;
	}

	// ֥Ȥ˾
	if ( !pFindTextObj->SetInfo( pDoc, pCursorInfo->GetCurPos(), rPattern, pConf ) )
		goto ERR_EXIT;

	// 
	if ( m_ProcessingShell->ShowDlg( pFindTextObj, ShowDlgDelay ) )
		goto ERR_EXIT;
	if ( !m_TaEditShell->ProcErrorMsg( pFindTextObj->GetErrorInfo() ) )
		goto ERR_EXIT;

	if ( !pFindTextObj->IsMatched() ) {
		// Ĥʤ
		m_TaEditShell->ShowInformationMsgBox( pConf->RefCantFindTextMsg() );
		goto ERR_EXIT;
	}

	// ֤
	nSelRange = pFindTextObj->GetMatchRange();
	nCurPos = nSelRange.GetEPos();
	pCursorInfo->SetCurPos( nCurPos, &scrPacket, nSelRange );

	// 
	if ( scrPacket.NeedXScroll || scrPacket.NeedYScroll )
		pDraw->AfterEditRedraw( scrPacket );
	else
		pDraw->AfterSelRangeRedraw( scrPacket.ChangeSLine, scrPacket.ChangeELine );

ERR_EXIT:
	delete pFindTextObj;
}


// ѥ˰פʸִ롣
void CTextEdit::ReplaceAll( const wstring &rPattern, const wstring& rAfterText )
{
	ScrollInfoPacket scrPacket;
	T_CurPos wCurPos;
	T_SelRange nSelRange;

	VClsPtr< CConfigInfo > pConf = m_TaEditShell->GetConfigInfo();
	CReplaceAll *pReplaceAllObj = new CReplaceAll();
	if ( NULL == pReplaceAllObj ) {
		m_TaEditShell->ShowErrorMsgBox( pConf->RefOutOfMemoryErrorMsg() );
		return ;
	}

	// ֥Ȥ˾
	if ( !pReplaceAllObj->SetInfo( pDoc, rPattern, pConf, rAfterText ) )
		goto ERR_EXIT;

	// 
//	m_ProcessingShell->ShowDlg( pReplaceAllObj );	// ֽץɽǰ

	// Ԥ
	pReplaceAllObj->start();
	pReplaceAllObj->WaitThreadTerminate();

	// 顼
	m_TaEditShell->ProcErrorMsg( pReplaceAllObj->GetErrorInfo() );

	// ֤
	wCurPos = T_CurPos( 0, 0 );	// ΥϡƥȤƬꤹ
	nSelRange = T_SelRange( wCurPos, wCurPos );
	pCursorInfo->SetCurPos( wCurPos, &scrPacket, nSelRange );

	// 
	pDraw->AfterEditRedraw( scrPacket );

ERR_EXIT:
	delete pReplaceAllObj;
}

// ʸΰưԤ
void CTextEdit::MoveText( T_SelRange SourceRange, int DestXPos, int DestYPos )
{
	ScrollInfoPacket scrPacket;
	wstring wStr;
	T_CurPos wCurPos;
	CErrorBool r;

	// ưоݤʸ
	pDoc->GetRangeStr( SourceRange, &wStr );

	// ֤׻
	T_CurPos DestPos = pDraw->PixPosToCurPos( DestXPos, DestYPos );

	// ֤ưξϡƤ롣
	if ( SourceRange.GetSPos() > DestPos ) {
		r = pDoc->InsertSpecialCode( XK_Delete, SourceRange.GetEPos(), SourceRange, &wCurPos );
		if ( r ) r = pDoc->Insert( wStr, DestPos, T_SelRange( DestPos, DestPos ), &wCurPos );
	}
	else {
		r = pDoc->Insert( wStr, DestPos, T_SelRange( DestPos, DestPos ), &wCurPos );
		if ( r ) r = pDoc->InsertSpecialCode( XK_Delete, SourceRange.GetEPos(), SourceRange, &wCurPos );
	}
	m_TaEditShell->ProcErrorMsg( r );

	// ֤ϰϤ
	T_SelRange nSelRange = T_SelRange( DestPos, DestPos );
	pCursorInfo->SetCurPos( DestPos, &scrPacket, nSelRange );

	// 
	pDraw->AfterEditRedraw( scrPacket );
}

// Ԥذư
// MoveToɽϤ줿Ԥذư
void CTextEdit::OnMoveto()
{
	ScrollInfoPacket scrPacket;
	T_CurPos wCurPos;
	int LineCnt = pDoc->RefData().size();	// ߤԿ
	VClsPtr< CConfigInfo > pConf = m_TaEditShell->GetConfigInfo();

	// ߤΥ֤
	int CurLine = pCursorInfo->GetCurPos().GetLine();
	CurLine += pConf->GetLineNumberOrigin();

	// ưιֹ
	int line = m_MovetoShell->ShowDlg( CurLine );
	line -= pConf->GetLineNumberOrigin();
	if ( line < 0 ) line = 0;
	if ( line >= LineCnt ) line = LineCnt - 1;

	// ư
	wCurPos.SetLine( line );
	wCurPos.SetCPos( 0 );
	pCursorInfo->SetCurPos( wCurPos, &scrPacket, false );
	pDraw->AfterEditRedraw( scrPacket );
}

// ϰ
void CTextEdit::ClearSelect()
{
	T_SelRange wSR;
	ScrollInfoPacket scrPacket;

	// ϰ򤵤Ƥϡϰƺ褹
	wSR = pCursorInfo->GetSelRange();
	if ( wSR.GetLength() == T_CurPos( 0, 0 ) )
		return ;

	const T_CurPos &wCP = pCursorInfo->GetCurPos();
	wSR.SetPos( wCP, wCP );
	pCursorInfo->SetCurPos( wCP, &scrPacket, wSR );
	pDraw->AfterSelRangeRedraw( scrPacket.ChangeSLine, scrPacket.ChangeELine );
}

